package yrpg.slm.pc.common;

import kd.bos.algo.DataSet;
import kd.bos.algo.Row;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.QueryServiceHelper;

import java.math.BigDecimal;

/**
 * 工资发放计算规则 返回计算金额
 * @author renxing
 */
public class GrantCommonRules {

    /**
     *  业绩奖发放规则
     * @param amount 业绩金额
     */
    public static BigDecimal grantAchievementRule(BigDecimal amount, DynamicObject scoreCol, DataSet allProjectScoreDataSet) {
        BigDecimal grantAmount = new BigDecimal(0);

        BigDecimal lastProgress = scoreCol.getBigDecimal("nowProgress");//本期科研进度
        //如果本期进度 达到百分百
        if (lastProgress.equals(new BigDecimal(100))) {

            BigDecimal grantBeforeAmount = new BigDecimal(0);
            //根据这个人这个科研项目获取所有打分
            if(allProjectScoreDataSet.hasNext()) {
                Row row = allProjectScoreDataSet.next();//
                BigDecimal detailAmount = row.getBigDecimal("realMoney");
                grantBeforeAmount = grantBeforeAmount.add(detailAmount);
            }
            //由于本次打分为100 所有本次实发金额为实发金额 减去之前所有实发金额乘实发比例之和  防止尾差
            grantAmount = amount.subtract(grantBeforeAmount);//

        } else {
            BigDecimal realMoney = scoreCol.getBigDecimal("realMoney");//获取实发比例
            //发放金额计算 为应发金额 * 实发比例 / 100
            grantAmount = amount.multiply(realMoney).divide(new BigDecimal(100));
        }


        return grantAmount;
    }

    public static BigDecimal grantAchievementRules(DataSet personProgressDataSet,DataSet nowGrantDataSet){

        BigDecimal grantAmount = BigDecimal.ZERO;
        BigDecimal compliance =BigDecimal.ZERO;//科研考核表
        BigDecimal amountOut = BigDecimal.ZERO;
        DynamicObjectCollection nowGrantCollection = ORM.create().toPlainDynamicObjectCollection(nowGrantDataSet.copy());
        for(DynamicObject nowGrantCol : nowGrantCollection){
            amountOut = nowGrantCol.getBigDecimal("beramountin");

        }
        DynamicObjectCollection personProgressCollection = ORM.create().toPlainDynamicObjectCollection(personProgressDataSet.copy());
        for(DynamicObject personProgressCol : personProgressCollection){
            compliance = personProgressCol.getBigDecimal("compliance");

        }
        //计算发放金额
        grantAmount = amountOut.multiply(compliance).divide(new BigDecimal(100));


        return grantAmount;
    }
    //行为奖 发放规则
    public static BigDecimal grantBehaviorRule(String billMark,DynamicObject period,DynamicObject person,DataSet nowPeriodDataSet){

        BigDecimal grantAmount = BigDecimal.ZERO;
        BigDecimal amountOut = BigDecimal.ZERO;
        if(nowPeriodDataSet.hasNext()){
            Row row = nowPeriodDataSet.next();
            amountOut = row.getBigDecimal("yrpg_beramount_out");

        }

        QFilter periodFilter = new QFilter("yrpg_period", QCP.equals, period);
        QFilter userFilter = new QFilter("yrpg_user", QCP.equals, person);
        DynamicObject awardData =
                QueryServiceHelper.queryOne(billMark, "id,yrpg_period,yrpg_user,yrpg_prize_entry.yrpg_complate_rate as rate", new QFilter[]{periodFilter,userFilter});
        BigDecimal rate = awardData.getBigDecimal("rate");//获取完成率

        //85目前为固定，应该获取基础资料配置信息
        if(rate.compareTo(new BigDecimal(85)) > 0){
            //超过  比例的部分式金额乘 完成率
            grantAmount = amountOut.multiply(rate).divide(new BigDecimal(100));
        }

        return grantAmount;
    }

    /**
     * 通用规则
     * @param nowPeriodDataSet
     * @return
     */
    public static BigDecimal grantNomalRule(DataSet nowPeriodDataSet){
        BigDecimal grantAmount = BigDecimal.ZERO;
        //遍历
        if (nowPeriodDataSet.hasNext()) {
            Row row = nowPeriodDataSet.next();//行
            BigDecimal amount = row.getBigDecimal("yrpg_beramount_out");//发出金额
            grantAmount = grantAmount.add(amount);
        }
        return grantAmount;

    }



}
