package yrpg.slm.pc.plugin.operate;

import kd.bos.algo.DataSet;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.BeforeOperationArgs;
import kd.bos.entity.plugin.args.BeginOperationTransactionArgs;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.sdk.plugin.Plugin;
import yrpg.slm.pc.common.GrantCommonRules;

import java.math.BigDecimal;

/**
 * renxing
 * 单据操作插件
 *   奖项保存时计算每个奖项的金额
 */
public class AwardDetailSaveOpPlugin extends AbstractOperationServicePlugIn implements Plugin {

    private DataSet grantDataSet = null;//工资发放单 该奖项数据
    private DataSet awardTypeDataSet = null;//所有奖项
    private DataSet projectScoreDataSet = null;//科研项目进度
    private DataSet nowBehaviorDataSet = null;//考核表

    private final String grantBill = "yrpg_wages_grant";//工资发放单
    private final String awardBill = "yrpg_pc_awardtype";//奖项
    private final String rulesBill = "yrpg_pc_data_rules";//取数规则
    private final String projectScoreBill = "yrpg_sc_projectscore";//科研项目打分表

    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        e.getFieldKeys().add("yrpg_srcid");
        e.getFieldKeys().add("yrpg_user");
        e.getFieldKeys().add("yrpg_period");
        e.getFieldKeys().add("yrpg_totalamount");
    }

    @Override
    public void beforeExecuteOperationTransaction(BeforeOperationArgs e) {

        this.getDataSet(grantBill);//将所有人的工资发放该奖项的数据源加载
        this.getDataSet(awardBill);//将奖项加载
        //this.getDataSet(rulesBill);//将取数规则加载
        //this.GetDataSet(projectScoreBill);//
        this.getDataSet("yrpg_behaviorscore");//加载考核表

        for (DynamicObject obj : e.getDataEntities()) {
            //获取当前单据的标识
            String billKey = obj.getDataEntityType().getName();
            //源单标识
            String srcid = obj.getString("yrpg_srcid");
            //获取当前人员
            DynamicObject person = obj.getDynamicObject("yrpg_user");
            //获取当前期间
            DynamicObject period = obj.getDynamicObject("yrpg_period");

            //奖项类型 由奖项发放标识获取
            DataSet nowAwardDataSet = awardTypeDataSet.copy().filter("yrpg_grand = " + "'" +billKey+ "'" );
            DynamicObjectCollection awardTypeCollection = ORM.create().toPlainDynamicObjectCollection(nowAwardDataSet.copy());

            if(awardTypeCollection.size() == 0){
                //找不到奖项
                continue;
            }
            long awardPkid = awardTypeCollection.stream().findFirst().get().getLong("id");
            //奖项数据源 工资发放单获取到当前单据头人员、单据体奖项过滤之后的DataSet
            DataSet nowGrantDataSet = grantDataSet.copy().filter("yrpg_user = " + person.getPkValue() + " and awardtype = " + awardPkid);

            //DynamicObjectCollection prizeEntry = obj.getDynamicObjectCollection("yrpg_prize_entry");//奖项单据体

            /**
             * 预留 发放规则计算  业绩奖通用计算方式
             */
            if(billKey.equals("yrpg_grant_achievement")) {

                //this.GrantAchievementGet(person,period,prizeEntry);
                //每个人科研进度合计
                DataSet personProgressDataSet = nowBehaviorDataSet.copy().filter("user = "+ person.getPkValue());
                BigDecimal grantAmount = GrantCommonRules.grantAchievementRules(personProgressDataSet,nowGrantDataSet);
                obj.set("yrpg_totalamount",grantAmount);//奖金计算赋值
            }

            /**
             * 预留 发放规则计算  达成奖 计算方式
             */
            else if (billKey.equals("")) {
                //获取工资计算奖项标识
                String  yrpgCompute = awardTypeCollection.stream().findFirst().get().getString("yrpg_compute");

                BigDecimal grantAmount = GrantCommonRules.grantBehaviorRule(billKey,period,person,nowGrantDataSet);
                obj.set("yrpg_totalamount",grantAmount);//奖金计算赋值
            }

            /**
             * 预留 计算普通奖项计算逻辑
             */
            else {
                //普通奖项金额计算
                BigDecimal grantAmount = GrantCommonRules.grantNomalRule(nowGrantDataSet);
                obj.set("yrpg_totalamount",grantAmount);//奖金计算赋值
            }


        }
    }

    public void getDataSet(String billMark){

        //期间取值  ？ "yrpg_period", 120230050);
        QFilter periodQf = new QFilter("yrpg_period", QCP.equals,120230050);//期间-期间年度查询条件：

        switch (billMark){
            case  "yrpg_wages_grant"://获取发放单
                //按照期间查询所有人在该会计期间内的工资发放单 该奖项数据
                grantDataSet = QueryServiceHelper.queryDataSet(this.getClass().toString(), billMark,
                        "yrpg_user,yrpg_period,yrpg_sourceentryentity1.yrpg_awardtype as awardtype,yrpg_sourceentryentity1.yrpg_beramount_in as beramountin", new QFilter[]{periodQf}, null);
                break;
            case "yrpg_pc_awardtype"://奖项 id、计算标识、发放标识、
                awardTypeDataSet = QueryServiceHelper.queryDataSet(this.getClass().toString(), billMark,
                        "id,yrpg_compute,yrpg_grand", null, null);
                break;
            case "yrpg_sc_projectscore"://科研项目进度表
                projectScoreDataSet = QueryServiceHelper.queryDataSet(this.getClass().toString(), billMark,
                        "id,yrpg_examine_period as period,yrpg_point_list.yrpg_scienceproject as scienceProject,yrpg_point_list.yrpg_username as user,yrpg_point_list.yrpg_now_progress as nowProgress" +
                                ",yrpg_point_list.yrpg_last_progress as lastProgress,yrpg_point_list.yrpg_realmoney as realMoney"
                        ,new QFilter[]{periodQf}, null);
                break;
            case "yrpg_behaviorscore"://考核表
                QFilter nowPeriodQf = new QFilter("yrpg_check_date", QCP.equals,120230050);
                nowBehaviorDataSet = QueryServiceHelper.queryDataSet(this.getClass().toString(), billMark,
                        "id,yrpg_check_date,yrpg_assessor.yrpg_check_userid as user,yrpg_assessor.yrpg_compliancetotal as compliance"
                        ,new QFilter[]{nowPeriodQf}, null);
                break;
            default:
                break;
        }


    }


    public void grantAchievementGet(DynamicObject person,DynamicObject period,DynamicObjectCollection prizeEntry){
        //获取当前人员当期 科研打分明细
        DataSet nowScoreDataSet = projectScoreDataSet.where("yrpg_user = " + person.getPkValue() + "and period = " + period); //科研项目进度表
        DynamicObjectCollection scoreCollection = ORM.create().toPlainDynamicObjectCollection(nowScoreDataSet.copy());

        BigDecimal amount = new BigDecimal(0);//获取实发金额
        //存在当期人员的打分表
        if (scoreCollection.getRowCount() > 0) {
            for (DynamicObject scoreCol : scoreCollection) {
                //获取当前人员 当前科研项目 所有期间  打分
                DataSet allProjectScoreDataSet = projectScoreDataSet.where("yrpg_user = " + person.getPkValue()  + "and scienceProject = " + scoreCol.getString("scienceProject")); //科研项目进度表

                DynamicObject prizeDynamicObject = prizeEntry.addNew();//新增 业绩奖单据体
                prizeDynamicObject.set("yrpg_scienceproject", scoreCol.getString("scienceProject"));//科研项目
                prizeDynamicObject.set("yrpg_beforescore", scoreCol.getBigDecimal("lastProgress"));//上期比例
                prizeDynamicObject.set("yrpg_nowscore", scoreCol.getBigDecimal("nowProgress"));//本期比例
                prizeDynamicObject.set("yrpg_difference", scoreCol.getBigDecimal("realMoney"));//实发比例
                //调用业绩奖取数规则
                BigDecimal grantAmount = GrantCommonRules.grantAchievementRule(amount,scoreCol,allProjectScoreDataSet);
                prizeDynamicObject.set("yrpg_entryamount", grantAmount);//实发金额
                prizeDynamicObject.set("yrpg_issuecycle", scoreCol.getString("period"));//科研项目
            }
        }
    }




}